#!/usr/bin/perl
# applies to Kantronics KPC3+ and KPC9612+ TNC's
# script to put the TNC into KISS mode
# reads the comm port number and baud rate from a text file named "kpc-port-config.txt"
# The text file must reside in the same folder with this script file.
# If the text file is missing, the user is prompted for the port and baud rate.
# Last update June 2, 2012 - Lee Bengston - K5DAT

use strict;
use warnings;

my ($count, $result, $cmdflag, $PortObj, $mycall, $comport);
my ($input, $portnum, $baudnum, $baudrate, @bauds, $bauds);
my $kiss1 = "INTFACE KISS\r";
my $kiss2 = "RESET\r";

@bauds = (1200, 2400, 4800, 9600, 19200);
my $confile = "kpc-port-config.txt";

my $nofile = 1;
if (-e $confile) {  # check for the existence of the config file
  $nofile = 0;
}

if ($nofile == 1)  {
  print "\n No configuration file found - please run the setup utility\n";
  print "in order to avoid being asked the next questions every time\n\n";
  sleep(3);

  print "Please enter the number corresponding to the serial port to which\nyour TNC is connected\n\n";
  print "1   COM1            7   COM7\n";
  print "2   COM2            8   COM8\n";
  print "3   COM3            9   COM9\n";
  print "4   COM4           10   COM10\n";
  print "5   COM5           11   COM11\n";
  print "6   COM6           12   COM12\n\n";
  print "> ";

  $input = 0;
  while ($input == 0)  {
    $portnum = <STDIN>;
    chomp $portnum;
    if ($portnum < 1)  {
      print "Invalid Selection\n\n";
    }
    elsif ($portnum > 12)  {
      print "Invalid Selection\n\n";
    }
    else {
      $input = 1;
      $comport = "COM$portnum";
    }
  }

  print "\n\nThe assumption is that Kantronics TNC's always\n";
  print "use 8 Data Bits, One Stop Bit, and No Parity\n\n";

  print "Please enter the number corresponding to the baud rate\nof your serial port\n\n";
  print "1  1200\n";
  print "2  2400\n";
  print "3  4800\n";
  print "4  9600\n";
  print "5  19200\n\n";
  print "> ";

  $input = 0;
  while ($input == 0)  {
    $baudnum = <STDIN>;
    chomp $baudnum;
    if ($baudnum < 1)  {
      print "Invalid Selection\n\n";
    }
    elsif ($baudnum > 5)  {
      print "Invalid Selection\n\n";
    }
    else {
      $input = 1;
      $baudnum = $baudnum - 1;
      $baudrate = $bauds[$baudnum];
    }
  }

}
else  {
  open CON, $confile;
  $comport = <CON>;
  chomp $comport;
  $baudrate = <CON>;
  chomp $baudrate;
  close(CON);
}
  
use Win32::SerialPort;

$PortObj = new Win32::SerialPort ($comport);

$PortObj->baudrate($baudrate);
$PortObj->parity("none");
$PortObj->databits(8);
$PortObj->stopbits(1);
$PortObj->handshake("rts");
$PortObj->write_settings;

$PortObj->read_char_time(0);
$PortObj->read_const_time(100);

# start with some commands to take it out of KISS mode in case it is.
# There's no good way detect it is already in KISS mode and leave it alone in that case

$PortObj->write(chr(192));
$PortObj->write(chr(255));
$PortObj->write(chr(192));

$PortObj->write(chr(003)); # control-c
$PortObj->write(chr(003));
$PortObj->write(chr(003));

# Next send some stuff trying to get the cmd prompt
$PortObj->write(chr(003));
$PortObj->write(chr(003));
$PortObj->write(chr(003)); # control-c
$PortObj->write("\r");

$PortObj->read(4096);
sleep (1);

$PortObj->write("\r");
$PortObj->read(4096); 
sleep (1);

$PortObj->write("\r\r"); # now send again and look for cmd prompt
# will iterate up to 10 times looking for the command prompt
# in case there's lots more in the buffer to read out
$count = 10; # choice of 10 was made after some trial and error
$cmdflag = 0;
while ($count > 0)  {
  $result = $PortObj->read(512);
  print "Reading Data...\n\n";
  if ($result =~ m/cmd:/)  {
    $cmdflag = 1;
    $count = 0;
  }
  else {
  $count = $count - 1;
  sleep(1);
  }
}

if ($cmdflag == 1)  {
  print "OK - got the cmd prompt\n\n";
}
else {
# send commands to take TNC out of KISS MODE - we'll try again just in case the
# first attempt didn't take

  print "no command prompt - sending commands to exit\nKISS mode in case that's the reason\n\n";

  $PortObj->write(chr(192));
  $PortObj->write(chr(255));
  $PortObj->write(chr(192));
  sleep (1);

  sleep(1);
  $PortObj->write(chr(003));
  $PortObj->write(chr(003));
  $PortObj->write(chr(003)); # control-c
  $PortObj->write("\r");

# check again and see if we can get command mode reponses
  $count = 15;
  while ($count > 0)  {
    $result = $PortObj->read(255);
    sleep(1);
    print "Reading data after sending commands to exit KISS mode...\n\n";
    if ($result =~ m/cmd:/)  {
      $cmdflag = 2;
      $count = 0;
      print "OK, saw the command prompt\n\n";
    }
#   if no command prompt, check for 'ground zero' mode
    elsif ($result =~ m/BAUD/)  {
    print "TNC found in 'ground zero' mode\n\n";
    $PortObj->write("*");
    sleep(1);
    $result = $PortObj->read(127);
    print "$result\n\n";
    print "Your TNC needs a callsign\nplease enter it below:\n";
    print " > ";
    $mycall = <STDIN>;
    chomp $mycall;
    $PortObj->write("$mycall\r");
    sleep(1);
    $result = $PortObj->read(63);
    print "$result\n\n";
    $cmdflag = 3;
    }
    else {
    $count = $count - 1;
    }
  }
}

if ($cmdflag == 0)  {
  print "Was never able to communicate\n\n";
}

if ($cmdflag > 0)  {

  $PortObj->write("XFLOW ON\r");
  sleep (1);
  $PortObj->write("FULLDUP OFF\r");
  sleep(1);

#  print "Sending commands to put the TNC in KISS mode\n\n";
# if we have a prompt, go ahead and put the TNC in KISS mode
  $PortObj->write($kiss1);
  sleep(1);
  $result = $PortObj->read(255);
#  print "$result\n\n";
  $PortObj->write($kiss2);
  sleep(1);
  $result = $PortObj->read(255);
#  print "$result\n\n";
  print "Commands were sent to put the TNC in KISS mode\n\n";
  sleep(1);
}
$PortObj->close;
